[GtkTemplate (ui = "/dev/geopjr/Tuba/ui/widgets/preview_card_internal.ui")]
public class Tuba.Widgets.PreviewCardInternal : Gtk.Box {
	[GtkChild] public unowned Gtk.Label author_label;
	[GtkChild] public unowned Gtk.Label title_label;
	[GtkChild] public unowned Gtk.Label description_label;
	[GtkChild] public unowned Gtk.Box internal_box;
}
