calendar, calendar>header {
	border: none;
}

calendar>header>button, calendar>grid>label {
	border-radius: 8px;
}

.notifications-all.dim-enabled .can-be-dimmed {
	opacity: 0.70;
}

.notification-badge {
	font-weight: bold;
	font-size: 0.8em;
	border-radius: 10px;
	min-width: 0.7em;
	padding: 2px 5px;
	background: var(--accent-bg-color);
	color: @text_view_bg;
}

.no-min-size {
	min-width: 0;
	min-height: 0;
}

.notification-badge.no-attention {
	background: alpha(var(--sidebar-fg-color), .4);
}

.font-bold {
	font-weight: bold;
}

.no-shadow {
	box-shadow: none;
}

headerbar.flat.no-title .title {
	opacity: 0;
}

.cover-badge {
	padding: 6px 12px;
	border-radius: 6px;
	margin: 8px;
}

.cover-badge.only-icon {
	padding: 6px;
}

/* .cover-badge.button > button {
	padding-top: 0px;
	padding-bottom: 0px;
} */

.ttl-flat-button {
	padding: 0px;
	margin: 0px;
}

.ttl-code {
	padding: 12px;
	background: rgba(150, 150, 150, .1);
	border-radius: 6px;
}

.italic {
	font-style: italic;
}

.monospace {
	font-family: monospace;
}

.ttl-label-emoji-no-click:hover {
	background: transparent;
}

.ttl-label-emoji-no-click {
	background: transparent;
	padding: 0;
}

.ttl-name-button {
	padding: 0 5px;
	margin-left: -5px;
	margin-right: -5px;
}

/*
	Profile
*/

.ttl-profile-cover .header-image {
	background: grey;
	border-radius: 0;
}

.ttl-profile-cover .main-avi avatar, .profile-editor avatar {
	border: 6px solid var(--dialog-bg-color);
	background: var(--dialog-bg-color);
}

.large .header-image {
	border-radius: 12px 12px 0 0;
}

/* Status action icons */
@keyframes rotate_star {
	from { -gtk-icon-transform: rotate(-72deg); }
	to {}
}

@keyframes bounce {
	0% { -gtk-icon-transform: translateY(-3px); }
	50% { -gtk-icon-transform: translateY(3px); }
	100% {}
}

@keyframes rotate_half {
	from { -gtk-icon-transform: rotate(-180deg); }
	to {}
}

.ttl-status-action-reblog.enabled image {
	animation: rotate_half 0.4s ease;
}

.ttl-status-action-bookmark.enabled image {
	animation: bounce 0.4s ease;
}

.ttl-status-action-star.enabled image {
	animation: rotate_star 0.4s ease;
}

.ttl-status-action-star:hover,
.ttl-status-action-star.enabled {
	color: var(--yellow-5);
}

.ttl-status-action-star:active {
	color: var(--yellow-4);
}

.ttl-status-action-reblog:hover,
.ttl-status-action-reblog.enabled {
	color: var(--purple-3);
}

.ttl-status-action-reblog:active {
	color: var(--purple-4);
}

.ttl-status-action-reply:hover,
.ttl-status-action-reply.enabled {
	color: var(--green-3);
}

.ttl-status-action-reply:active {
	color: var(--green-4);
}

.ttl-status-action-bookmark:hover,
.ttl-status-action-bookmark.enabled {
	color: var(--red-3);
}

.ttl-status-action-bookmark:active {
	color: var(--red-4);
}

.ttl-status-action-quote:hover,
.ttl-status-action-quote.enabled {
	color: var(--blue-3);
}

.ttl-status-action-quote:active {
	color: var(--blue-4);
}

.ttl-profile-stat-button {
	padding: 8px 12px;
	min-height: 32px;
	border-radius: 0px;
}

.ttl-profile-stat-button:first-child {
	border-bottom-left-radius: 12px;
}

.ttl-profile-stat-button:last-child {
	border-bottom-right-radius: 12px;
}

.small .ttl-profile-stat-button:first-child,
.small .ttl-profile-stat-button:last-child {
	border-bottom-left-radius: 0px;
	border-bottom-right-radius: 0px;
}

.small .ttl-profile-cover>.content {
	border-bottom-left-radius: 0px;
	border-bottom-right-radius: 0px;
}

.ttl-box-no-shadow>revealer>box {
	box-shadow: none;
}

.ttl-status-font-large {
	font-size: larger;
}

.ttl-status-font-large > .ttl-large-body {
	font-size: larger;
}

.ttl-large-body {
	font-size: larger;
}

.ttl-status-line-height-large {
	line-height: 1.4;
}

.ttl-status-badge {
	border-radius: 6px;
	margin: 8px;
	padding: 1px 6px;
}

.ttl-status-badge:not(.error):not(.success) {
	color: white;
	background-color: rgba(0, 0, 0, .8);
}

.ttl-status-badge.sensitive-label {
	padding: 10px 32px;
	border-radius: 9999px;
}

.ttl-profile-fields-box-container:not(.single) > .ttl-profile-fields-box > flowboxchild > .ttl-profile-field {
	border-bottom: solid 1px var(--border-color);
	border-right: solid 1px var(--border-color);
}

.ttl-profile-fields-box-container.odd > .ttl-profile-fields-box > flowboxchild:last-child > .ttl-profile-field {
	border-bottom: none;
}

.ttl-profile-fields-box-container:not(.odd) {
	border: none;
}

.ttl-profile-fields-box-container:not(.single) {
	margin-right: -1px; /* much simpler than complex nth-child selectors */
}

.ttl-profile-field {
	padding: 6px 12px;
}

.ttl-verified-field {
	color: var(--success-color);
	background: alpha(var(--success-bg-color), 0.1);
}

.ttl-birthday-field {
	color: var(--accent-pink);
	background: alpha(var(--accent-pink), 0.1);
}

.media-viewer {
	color: white;
}

.media-viewer .top-bar {
	background-image: none;
}

.ttl-status-heading {
	font-size: small;
}

.ttl-status-heading-padding {
	margin-top: -5px;
	padding: 0 5px;
	margin-left: -5px;
	margin-right: -5px;
}

.ttl-mutuals {
	font-size: small;
	padding: 0 5px;
	margin-left: -5px;
	margin-right: -5px;
}

.ttl-mutuals label {
	font-weight: normal;
}

.ttl-mutuals .mutual-avi:not(.first-avi) {
	margin-left: -6px;
}

.ttl-status-avatar-actor {
	margin-top: -20px;
	margin-left: -6px;
	padding: 0;
	background: none;
}

.ttl-status-avatar-border {
	border: 3px solid var(--popover-bg-color);
	border-radius: 100%;
	background-color: var(--popover-bg-color);
}

.card-spacing {
	margin-top: 6px;
	margin-bottom: 6px;
}

.card-spacing > *:focus-within {
	border-radius: inherit;
}

.ttl-post.direct {
	background-color: alpha(var(--warning-bg-color), .1);
}

.ttl-post.direct:hover {
	background-color: darker(alpha(var(--warning-bg-color), .1));
}

.ttl-post-actions {
	margin-bottom: -12px;
}

.ttl-quote {
	border-radius: 6px;
	font-weight: initial;
}

.attachment-overlay-icon {
	border-radius: 100%;
	min-height: 64px;
	min-width: 64px;
}

.attachment-picture {
	border-radius: 6px;
}

.about .app-version{color:#fff;text-shadow:-1px -1px 0#000,1px -1px 0#000,-1px 1px 0#000,1px 1px 0#000;background:linear-gradient(150deg,#fdd81755 0,#66338b55 8.3%,#fff5 16.6%,#f4aec855 24.9%,#7bcce555 33.2%,#94551655 41.5%,#0005 49.8%,#e2201655 58.1%,#f2891755 66.4%,#efe52455 74.7%,#78b82a55 83%,#2c58a4 91.3%,#6d238055 100%);}
.about .app-version:hover{background:radial-gradient(circle at 9.75% 50%,#0000 6.66%,#7902aa 6.7%,#0000 8.4%),conic-gradient(at 26.66% 50%,#0000 222.75deg,#ffd800 0 317.25deg,#0000 0),conic-gradient(at 33% 50%,#0000 222.75deg,white 0 317.25deg,#0000 0),conic-gradient(at 39% 50%,#0000 222.75deg,#ffa6b9 0 317.25deg,#0000 0),conic-gradient(at 45.66% 50%,#0000 222.75deg,#00d2ff 0 317.25deg,#0000 0),conic-gradient(at 52% 50%,#0000 222.75deg,#753000 0 317.25deg,#0000 0),conic-gradient(at 58.33% 50%,#0000 222.75deg,#000 0 317.25deg,#0000 0),linear-gradient(to bottom,#e40303,#e40303 16.67%,#ff8c00 16.67%,#ff8c00 33.33%,#ffed00 33.33%,#ffed00 50%,#008026 50%,#008026 66.67%,#004dff 66.67%,#004dff 83.33%,#750787 83.33%,#750787);}

/* Composer auto completion */
GtkSourceAssistant.completion list row {
	padding: 6px 12px 6px 12px;
}

.no-border-radius {
	border-radius: 0px;
}

.locale-dropdown-upcase {
	text-transform: uppercase;
}

flowboxchild,
.no-padding {
	padding: 0;
}

.preview_card {
	padding: 0;
}

.preview_card_v {
	border-radius: 6px 6px 0px 0px;
}

.preview_card_h:dir(ltr) {
	border-radius: 6px 0px 0px 6px;
}

.preview_card_h:dir(rtl) {
	border-radius: 0px 6px 6px 0px;
}


.bkwm-desc {
	padding: 12px;
}

video > overlay > revealer > controls, .audio-controls {
	padding: 12px;
}

.spoilered-attachment {
	filter: blur(30px);
}

.lww-line-height {
	line-height: 1.8;
	margin-top: -5px;
	margin-bottom: -5px;
}

.theme-disability.about .app-version{background:linear-gradient(45deg,#59595955 0,#59595955 37.5%,#cf728055 37.5%,#cf728055 42.5%,#eede7755 42.5%,#eede7755 47.5%,#e8e8e855 47.5%,#e8e8e855 52.5%,#7bc2e055 52.5%,#7bc2e055 57.5%,#3bb07d55 57.5%,#3bb07d55 62.5%,#59595955 62.5%,#59595955 100%);}
.theme-disability.about .app-version:hover{background:linear-gradient(45deg,#595959 0,#595959 37.5%,#cf7280 37.5%,#cf7280 42.5%,#eede77 42.5%,#eede77 47.5%,#e8e8e8 47.5%,#e8e8e8 52.5%,#7bc2e0 52.5%,#7bc2e0 57.5%,#3bb07d 57.5%,#3bb07d 62.5%,#595959 62.5%,#595959 100%);}
.theme-lesbian.about .app-version{background:linear-gradient(to bottom,#d1260155,#d1260155 20%,#ff8f4555 20%,#ff8f4555 40%,#fff5 40%,#fff5 60%,#cf559b55 60%,#cf559b55 80%,#9b1a5855 80%,#9b1a5855);}
.theme-lesbian.about .app-version:hover{background:linear-gradient(to bottom,#d12601,#d12601 20%,#ff8f45 20%,#ff8f45 40%,#fff 40%,#fff 60%,#cf559b 60%,#cf559b 80%,#9b1a58 80%,#9b1a58);}
.theme-pan.about .app-version{background:linear-gradient(to bottom,#ff218c55,#ff218c55 33.33%,#fed80055 33.33%,#fed80055 66.67%,#21b1ff55 66.67%,#21b1ff55);}
.theme-pan.about .app-version:hover{background:linear-gradient(to bottom,#ff218c,#ff218c 33.33%,#fed800 33.33%,#fed800 66.67%,#21b1ff 66.67%,#21b1ff);}
.theme-trans.about .app-version{background:linear-gradient(to bottom,#5ccefa55,#5ccefa55 20%,#f6a8b755 20%,#f6a8b755 40%,#fff5 40%,#fff5 60%,#f6a8b755 60%,#f6a8b755 80%,#5ccefa55 80%,#5ccefa55);}
.theme-trans.about .app-version:hover{background:linear-gradient(to bottom,#5ccefa,#5ccefa 20%,#f6a8b7 20%,#f6a8b7 40%,#fff 40%,#fff 60%,#f6a8b7 60%,#f6a8b7 80%,#5ccefa 80%,#5ccefa);}
.theme-bi.about .app-version{background:linear-gradient(to bottom,#d6027055,#d6027055 40%,#9a4f9655 40%,#9a4f9655 60%,#0039a955 60%,#0039a955);}
.theme-bi.about .app-version:hover{background:linear-gradient(to bottom,#d60270,#d60270 40%,#9a4f96 40%,#9a4f96 60%,#0039a9 60%,#0039a9);}
.theme-agender.about .app-version{background:linear-gradient(to bottom,#0005,#00000055 14.29%,#bcc4c755 14.29%,#bcc4c755 28.57%,#fff5 28.57%,#fff5 42.86%,#b7f58455 42.86%,#b7f58455 57.14%,#fff5 57.14%,#fff5 71.43%,#bcc4c755 71.43%,#bcc4c755 85.71%,#00000055 85.71%,#0005);}
.theme-agender.about .app-version:hover{background:linear-gradient(to bottom,#000,#000 14.29%,#bcc4c7 14.29%,#bcc4c7 28.57%,#fff 28.57%,#fff 42.86%,#b7f584 42.86%,#b7f584 57.14%,#fff 57.14%,#fff 71.43%,#bcc4c7 71.43%,#bcc4c7 85.71%,#000 85.71%,#000);}
.theme-aro.about .app-version{background:linear-gradient(to bottom,#309c3455,#309c3455 20%,#9dce6955 20%,#9dce6955 40%,#fff5 40%,#fff5 60%,#a0a0a055 60%,#a0a0a055 80%,#0005 80%,#0005);}
.theme-aro.about .app-version:hover{background:linear-gradient(to bottom,#309c34,#309c34 20%,#9dce69 20%,#9dce69 40%,#fff 40%,#fff 60%,#a0a0a0 60%,#a0a0a0 80%,#000 80%,#000);}
.theme-ace.about .app-version{background:linear-gradient(to bottom,#0005,#0005 25%,#a3a3a355 25%,#a3a3a355 50%,#fff5 50%,#fff5 75%,#82008055 75%,#82008055);}
.theme-ace.about .app-version:hover{background:linear-gradient(to bottom,#000,#000 25%,#a3a3a3 25%,#a3a3a3 50%,#fff 50%,#fff 75%,#820080 75%,#820080);}
.theme-non-binary.about .app-version{background:linear-gradient(to bottom,#fdf52f55,#fdf52f55 25%,#fff5 25%,#fff5 50%,#9d5ad255 50%,#9d5ad255 75%,#0005 75%,#0005);}
.theme-non-binary.about .app-version:hover{background:linear-gradient(to bottom,#fdf52f,#fdf52f 25%,#fff 25%,#fff 50%,#9d5ad2 50%,#9d5ad2 75%,#000 75%,#000);}
.theme-black-history.about .app-version{background:linear-gradient(to bottom,#e31b2355,#e31b2355 33%,#0005 33%,#0005 66%,#00853f55 66%,#00853f55);}
.theme-black-history.about .app-version:hover{background:linear-gradient(to bottom,#e31b23,#e31b23 33%,#000 33%,#000 66%,#00853f 66%,#00853f);}
.theme-aids.about .app-version{background:linear-gradient(to bottom,#ffffff55,#ffffff55 20%,#d1111355 20%,#d1111355 40%,#ffffff55 40%,#ffffff55 60%,#d1111355 60%,#d1111355 80%,#ffffff55 80%,#ffffff55);}
.theme-aids.about .app-version:hover{background:linear-gradient(to bottom,#fff,#fff 20%,#d11113 20%,#d11113 40%,#fff 40%,#fff 60%,#d11113 60%,#d11113 80%,#fff 80%,#fff);}
.theme-intersex.about .app-version{background:radial-gradient(closest-side circle at center,#ffd80055 44%,#7902aa55 44%,#7902aa55 56%,#ffd80055 56%);}
.theme-intersex.about .app-version:hover{background:radial-gradient(closest-side circle at center,#ffd800 44%,#7902aa 44%,#7902aa 56%,#ffd800 56%);}
.theme-autism.about .app-version{background:linear-gradient(to bottom,#e5304355,#e5304355 16.66%,#f6834755 16.66%,#f6834755 33.33%,#fac85755 33.33%,#fac85755 66.67%,#99d67155 66.67%,#99d67155 83.33%,#21ae7555 83.33%,#21ae7555);}
.theme-autism.about .app-version:hover{background:linear-gradient(to bottom,#e53043,#e53043 16.66%,#f68347 16.66%,#f68347 33.33%,#fac857 33.33%,#fac857 66.67%,#99d671 66.67%,#99d671 83.33%,#21ae75 83.33%,#21ae75);}

.lww-scale-emoji-hover .lww-emoji {
	transition-duration: 150ms;
}

.lww-scale-emoji-hover .lww-emoji:hover {
	transform: scale(2);
}

@keyframes loadbg {
	0% { background-size: 0%; }
	95% { background-size: 105%; }
	100% { background-size: 100%; }
}

.ttl-poll-voted {
	animation: loadbg 0.5s ease;
}

.ttl-poll-row:first-child {
	border-top-left-radius: 12px;
	border-top-right-radius: 12px;
}

.ttl-poll-row:last-child {
	border-bottom-left-radius: 12px;
	border-bottom-right-radius: 12px;
}

.picker-emoji-button {
	padding: 5px;
}

.background-none {
	background: none;
}

.profile-editor .background-none text {
	background: none;
}

.profile-editor .background-cover {
	border-radius: 12px;
	background-color: var(--shade-color);
}

.media-viewer-fab {
	min-width: 42px;
	min-height: 42px;
}

.circular-spinner {
	border-radius: 100%;
	padding: 3px;
}

.attention {
	color: var(--yellow-4);
}

.ttl-view listview>row.activatable:selected:hover,
.ttl-view listview>row.activatable:hover {
	background-color: transparent;
}

.ttl-view listview>row.activatable:selected:active,
.ttl-view listview>row.activatable:active {
	box-shadow: none;
}

/* .ttl-view .content row:first-child { */
.ttl-view .fake-content row:first-child {
	margin-top: 0px;
}

/* .ttl-view .content row:last-child { */
.ttl-view .fake-content row:last-child {
	margin-bottom: 0px;
}

/* .ttl-view:not(.large-view) .content .card { */
.ttl-view:not(.large-view) .fake-content .card,
.ttl-view:not(.large-view) .fake-content .toggle-group-17 {
	border-left: none;
	border-right: none;
	border-radius: 0px;
}

/* .ttl-view:not(.large-view) .content { */
.ttl-view:not(.large-view) .fake-content {
	padding: 0px;
}

/* .ttl-view .content row:not(.ttl-post) { */
.ttl-view .fake-content row:not(.ttl-post):not(.toggle-group-17) {
	padding: 0px;
}

.card-spacing {
	background-color: var(--card-bg-color);
}

.ttl-profile-cover > .content {
	background: none;
}

/* .ttl-view .content { */
.ttl-view .fake-content {
	transition: all 200ms cubic-bezier(0.25, 0.46, 0.45, 0.94);
}

.ttl-view.large-view .fake-content:first-child {
	padding-top: 32px;
}

.ttl-view.large-view .fake-content:last-child {
	padding-bottom: 32px;
}

/* .ttl-view:not(.large-view) .content .card { */
.ttl-view:not(.large-view) .fake-content .card {
	border-bottom: 1px var(--border-color) solid;
}

/* .ttl-view:not(.large-view) .content .card:first-child { */
.ttl-view:not(.large-view) .fake-content .card:first-child {
	border-top: 1px var(--border-color) solid;
}

/* .ttl-view:not(.large-view) .content .card { */
.ttl-view:not(.large-view) .fake-content .card {
	box-shadow: none;
}

.ttl-view:not(.large-view) .preview_card.explore .preview_card_h {
	border-radius: 0px;
}

.ttl-view:not(.large-view) .preview_card.explore {
	border-radius: 0px;
}

.ttl-view .preview_card.explore {
	border-radius: 12px;
}

/* .ttl-view:not(.large-view) .content .card.card-spacing { */
.ttl-view:not(.large-view) .fake-content .card.card-spacing {
	margin: 0px;
}

.compose-button {
	padding: 8px;
}

.sidebar-item {
	padding: 3px 14px;
}

.ttl-view.large-view .fake-content {
	padding-left: 0;
	padding-right: 0;
	margin-left: 18px;
	margin-right: 18px;
}

.ttl-separator {
	background-color: transparent;
	min-height: 8px;
}

.ttl-account-switcher {
	padding: 0;
}

.ttl-account-switcher row {
	border-radius: 10px;
	margin: 3px 0px;
	padding-right: 0px;
	padding-left: 0px;
}

.ttl-account-switcher row:first-child {
	margin-top: 0px;
}

.ttl-account-switcher .new-account {
	margin-bottom: 0px;
}

.ttl-account-switcher separator {
	margin: 3px 0px;
}

.ttl-account-switcher row:selected avatar {
	border: 2px solid var(--accent-bg-color);
}

.ttl-account-switcher .blue-checkmark {
	color: var(--light-1);
	border-radius: 9999px;
	border: solid var(--accent-bg-color) 2px;
	background-color: var(--accent-bg-color);
	margin-bottom: 6px;
	opacity: 0;
}

.ttl-account-switcher row:selected .blue-checkmark {
	opacity: 1;
}

.scroll-to-top-btn {
	transition-duration: 200ms;
	transition-property: margin-bottom;
}

.scroll-to-top-btn.composer-btn-revealed {
	margin-bottom: 56px;
}

.profile-role {
	background-color: alpha(currentColor, .1);
	padding: 4px 11px;
	font-size: 12.2px;
	font-weight: 500;
}

.profile-role-border-radius {
	border-radius: 9999px;
}

.hashtag-bar-hashtag {
	padding: 2px 7px;
	font-size: small;
}

.preview_card:not(.explore),
.preview_card:not(.explore) image:not(.attachment-overlay-icon) {
	background-color: alpha(currentColor, 0.0625);
}

.preview_card:not(.explore):hover {
	background-color: alpha(currentColor, 0.125);
}

.preview_card:not(.explore):active {
	background-color: alpha(currentColor, 0.1625);
}

GtkSourceAssistant {
	background: var(--popover-bg-color);
	padding: 8px;
	/* border-radius of child + padding */
	border-radius: 14px;

}

GtkSourceAssistant scrolledwindow undershoot {
	background-image: none;
	box-shadow: none;
}

GtkSourceAssistant scrolledwindow .cell.icon {
	padding-left: 0;
	padding-top: 0;
	padding-bottom: 0;
}

GtkSourceAssistant row {
	border-radius: 6px;
	margin: 6px 0;
	padding: 6px;
}

GtkSourceAssistant row:selected {
	background: alpha(var(--accent-bg-color), .33);
	color: currentColor;
}

GtkSourceAssistant row:first-child,
GtkSourceAssistant row:last-child {
	margin: 0;
}

.account-avatar {
	margin-top: -30px;
}

.account-stats {
	font-size: small;
}

.filter {
	margin: -2px;
	border-radius: 0;
}

.sourceview.reset {
	background: transparent;
	color: currentColor;
}

.entry-button {
	margin: 0;
	padding: 0;
	min-width: 0; /* 24px; */
	min-height: 0; /* 24px; */
}

.entry-button:not(:hover):not(:focus-within) > image {
	opacity: 0.7;
}

popover.mini-profile > contents {
	padding: 0;
}

.report-status .attachment-picture {
	filter: blur(15px) grayscale(1);
}

.report-status:hover .attachment-picture {
	filter: none;
}

.focuspickerdialog .last-row {
	border-bottom-left-radius: 10px;
	border-bottom-right-radius: 10px;
}

.focuspickerdialog .focus-picker {
	border-radius: 9999px;
	padding: 4px;
}

.verified-author {
	padding: 0 5px;
	font-size: small;
}

list.uniform-border-color row {
	border-bottom-color: var(--border-color);
}

.emoji-reaction-expander>box>list,
.emoji-reaction-expander>box>list>row {
	border-radius: inherit;
}

.ttl-profile-cover.thanks {
	background: alpha(var(--accent-bg-color), 0.1);
}

.ttl-profile-cover.thanks avatar {
	border-color: alpha(var(--accent-bg-color), 0.1);
}

.toggle-group-17 {
	background: color-mix(in srgb, currentColor 10%, transparent);
	border-radius: 9px;
	padding: 3px;
}

.toggle-group-17 button {
	padding: 2px 15px;
	transition: 200ms cubic-bezier(0.25, 0.46, 0.45, 0.94);
	transition-property: outline-color, outline-width, outline-offset, background, box-shadow;
}

.toggle-group-17 button:checked {
	background: rgb(from var(--card-bg-color) r g b / calc(alpha * 2));
	color: var(--card-fg-color);
	box-shadow: 0 1px 3px 1px rgb(0 0 0 / 7%),
		0 2px 6px 2px rgb(0 0 0 / 3%);
}

.toggle-group-17 separator {
	margin: 3px 1px;
	transition: opacity 200ms cubic-bezier(0.25, 0.46, 0.45, 0.94);
}

.toggle-group-17 separator.hidden {
	opacity: 0;
}

@keyframes annual-avi-bg {
	0% { background-position: center 100%; }
	100% { background-position: center 0%; }
}

@keyframes annual-tuba-bg {
	0% { background-position: 0%; }
	100% { background-position: 100%; }
}

.annual .main-avi {
	background: -gtk-icontheme("tuba-explore2-large-symbolic");
	background-repeat: repeat;
	background-size: 16px;
	padding: 16px;
	animation: annual-avi-bg 5s linear infinite;
}

.annual .tuba {
	background-image: -gtk-icontheme("tuba-transparent-symbolic");
	background-repeat: repeat;
	background-size: 24px;
	background-position: center center;
	color: var(--accent-yellow);
	background-color: color-mix(in srgb, currentColor 15%, var(--window-bg-color));
	animation: annual-tuba-bg 5s linear infinite;
}

.annual-screenshot-box {
	padding: 12px;
	border-radius: 16px;
}

.annual-screenshot-box.style-window {
	background-color: var(--window-bg-color);
}

.annual-screenshot-box.style-accent {
	background-color: var(--accent-bg-color);
}

.annual-screenshot-box.style-pride {
	background-image: linear-gradient(135deg, rgba(228,3,3,0.3) 0%, rgba(255,140,0,0.3) 20%, rgba(255,237,0,0.3) 40%, rgba(0,128,38,0.3) 60%, rgba(0,77,255,0.3) 80%, rgba(117,7,135,0.3) 100%);
	background-color: var(--window-bg-color);
}

.annual-screenshot-box.style-trans {
	background-image: linear-gradient(135deg, rgba(92,206,250,0.5) 0%, rgba(246,168,183,0.5) 25%, rgba(255,255,255,0.5) 50%, rgba(246,168,183,0.5) 75%, rgba(92,206,250,0.5) 100%);
	background-color: var(--window-bg-color);
}
