.ttl-status-action-star:hover,
.ttl-status-action-star.enabled {
	color: var(--yellow-1);
}

.ttl-status-action-star:active {
	color: var(--yellow-2);
}

.ttl-status-action-reblog:hover,
.ttl-status-action-reblog.enabled {
	color: var(--purple-1);
}

.ttl-status-action-reblog:active {
	color: var(--purple-2);
}

.ttl-status-action-reply:hover,
.ttl-status-action-reply.enabled {
	color: var(--green-1);
}

.ttl-status-action-reply:active {
	color: var(--green-2);
}

.ttl-status-action-bookmark:hover,
.ttl-status-action-bookmark.enabled {
	color: var(--red-1);
}

.ttl-status-action-bookmark:active {
	color: var(--red-2);
}

.ttl-status-action-quote:hover,
.ttl-status-action-quote.enabled {
	color: var(--blue-1);
}

.ttl-status-action-quote:active {
	color: var(--blue-2);
}

.ttl-post.direct:hover {
	background-color: lighter(alpha(var(--warning-bg-color), .1));
}

.ttl-darken-images avatar:not(:hover), .ttl-darken-images picture:not(:hover) {
	filter: brightness(.8) contrast(1.2);
}
